@@ -5,4 +5,4 @@ from .evolution import Evolution
 from .creatures import EvogressionCreature
 from . import standardize
 from . import groups
-from .groups import evolution_group, random_population, Population
+from .groups import evolution_group, parameter_usage, output_usage, random_population, Population
