@@ -10,6 +10,7 @@ import tqdm
 import warnings
 from collections import defaultdict
 import easy_multip
+import pickle
 from pandas import DataFrame
 from .creatures import EvogressionCreature
 from .standardize import Standardizer
@@ -21,6 +22,9 @@ except ImportError:
     print('See: https://wiki.python.org/moin/WindowsCompilers')
     print('  -> (If running Windows 7, try using Python 3.7 instead of 3.8+)\n')
 
+# function must be module-level so that it is pickleable for multiprocessing
+# making it a global variable that is a function set in evolution __init__ to be module-level but still customizable
+find_best_creature_multip = None
 
 
 class BaseEvolution():
@@ -40,12 +44,20 @@ class BaseEvolution():
                  force_num_layers: int=0,
                  max_layers: int=10,
                  standardize: bool=True,
-                 use_multip: bool=True,
+                 num_cpu: int=1,
                  fill_none: bool=True,
+                 verbose: bool=True,
                  **kwargs) -> None:
 
+        if 'use_multip' in kwargs:
+            print('\nDeprecationWarning!  Instead of "use_multip", use "num_cpu" to specify how many processes.')
+            if kwargs['use_multip'] == True:
+                print('Using 2 processes for now.')
+                num_cpu = 2
+
         self.target_parameter = target_parameter
         self.standardize = standardize
+        self.verbose = verbose
 
         if type(all_data) == DataFrame:
             self.all_data = all_data.to_dict('records')
@@ -58,7 +70,9 @@ class BaseEvolution():
         self.num_cycles = int(num_cycles)
         self.force_num_layers = int(force_num_layers)
         self.max_layers = max_layers
-        self.use_multip = use_multip
+        self.num_cpu = num_cpu if num_cpu >= 1 else 1
+        global find_best_creature_multip
+        find_best_creature_multip = easy_multip.decorators.use_multip(find_best_creature, num_cpu=self.num_cpu)
 
         self.best_creatures: list = []
         self.parameter_usefulness_count: dict=defaultdict(int)
@@ -143,14 +157,16 @@ class BaseEvolution():
         counter = 0
         while counter < self.num_cycles:
             counter += 1
-            print('----------------------------------------' + f'\nCycle - {counter} -')
+            if self.verbose:
+                print('----------------------------------------' + f'\nCycle - {counter} -')
 
             best_creature, error, median_error = self.calculate_all_and_find_best_creature(progressbar=progressbar)
             self.current_median_error = median_error
-            self.print_cycle_stats(best_creature=best_creature, error=error, median_error=median_error, best_creature_error=error)
+            if self.verbose:
+                self.print_cycle_stats(best_creature=best_creature, error=error, median_error=median_error, best_creature_error=error)
 
-            if self.record_best_creature(best_creature, error):
-                print(f'\n\n\nNEW BEST CREATURE AFTER {counter} ITERATION{"S" if counter > 1 else ""}...')
+            if self.record_best_creature(best_creature, error) and self.verbose:
+                print(f'\n\nNEW BEST CREATURE AFTER {counter} ITERATION{"S" if counter > 1 else ""}...')
                 print(best_creature)
                 print('Total Error: ' + '{0:.2E}'.format(error))
 
@@ -251,7 +267,7 @@ class BaseEvolution():
         Find the best creature in all current creatures by calculating each one's
         total error compared to all the training data.
         '''
-        if self.use_multip:
+        if self.num_cpu > 1:
             if self.standardize:
                 result_data = find_best_creature_multip(self.creatures, self.target_parameter,
                                                                              self.standardized_all_data, standardizer=self.standardizer,
@@ -267,7 +283,7 @@ class BaseEvolution():
 
 
     def print_cycle_stats(self, best_creature=None, error=None, median_error: float=0, best_creature_error: float=0) -> None:
-        print('Median error: ' + '{0:.2E}'.format(median_error))
+        print('\rMedian error: ' + '{0:.2E}'.format(median_error))
         print('Best Creature:')
         print(f'  Generation: {best_creature.generation}    Error: ' + '{0:.2E}'.format(error))
 
@@ -295,35 +311,41 @@ class BaseEvolution():
         Use the creature.mutate_to_new_creature method to
         transform the best_creature into an even better fit.
         '''
-        print('\n\n\nOptimizing best creature...')
         best_creature = self.best_creature
-        print(best_creature)
+        if self.verbose:
+            print('\n\n\nOptimizing best creature...')
+            print(best_creature)
         errors = []
         adjustments = 'fast'  # start out with larger, faster mutations
-        for i in tqdm.tqdm(range(iterations)):
-            if i > iterations / 3:  # quickly mutate creature for first 1/3rd of iterations and then make small, fine mutations
-                adjustments = 'fine'
-
-            mutated_clones = [best_creature] + [best_creature.mutate_to_new_creature(adjustments=adjustments) for _ in range(500)]
-
-            if self.use_multip:
-                result_data = find_best_creature_multip(mutated_clones, self.target_parameter, self.standardized_all_data, progressbar=False)
-                best_creature, error, median_error, calculated_creatures = self.stats_from_find_best_creature_multip_result(result_data)
-            else:
-                best_creature, error, median_error, calculated_creatures = find_best_creature(mutated_clones, self.target_parameter, self.standardized_all_data, progressbar=False)
-
-            print('Best error: ' + '{0:.6E}'.format(error))
-            errors.append(error)
-            if error == 0:
-                break  # break out of loop if no error/perfect regression
-            if i > 5 and error / errors[-3] > 0.9999:
-                if adjustments == 'fast':
+        with tqdm.tqdm(total=iterations, position=1, bar_format='{desc}', desc='...') as desc:
+            for i in tqdm.tqdm(range(iterations), desc='Optimizing', position=0, leave=True):
+                if i > iterations / 3:  # quickly mutate creature for first 1/3rd of iterations and then make small, fine mutations
                     adjustments = 'fine'
+
+                mutated_clones = [best_creature] + [best_creature.mutate_to_new_creature(adjustments=adjustments) for _ in range(500)]
+
+                if self.num_cpu > 1:
+                    result_data = find_best_creature_multip(mutated_clones, self.target_parameter, self.standardized_all_data, progressbar=False)
+                    best_creature, error, median_error, calculated_creatures = self.stats_from_find_best_creature_multip_result(result_data)
                 else:
-                    break  # break out of the loop if it's no longer improving accuracy
+                    best_creature, error, median_error, calculated_creatures = find_best_creature(mutated_clones, self.target_parameter, self.standardized_all_data, progressbar=False)
+
+                errors.append(error)
+                if self.verbose:
+                    # print('\nBest error: ' + '{0:.6E}'.format(error))
+                    desc.set_description(f'Start: {errors[0]:.5E}     Best: {error:.5E}')
+
+                if error == 0:
+                    break  # break out of loop if no error/perfect regression
+                if i > 5 and error / errors[-3] > 0.9999:
+                    if adjustments == 'fast':
+                        adjustments = 'fine'
+                    else:
+                        break  # break out of the loop if it's no longer improving accuracy
         self.record_best_creature(best_creature, error)
+        print('\nBest creature optimized!')
         print(self.best_creature)
-        print('Best creature optimized!\n')
+
 
 
     def output_best_regression(self, output_filename='regression_function', add_error_value=False) -> None:
@@ -340,6 +362,36 @@ class BaseEvolution():
             self.best_creature.output_python_regression_module(output_filename=output_filename, directory='regression_modules', name_ext=name_ext)
 
 
+    def save(self, filename='evolution_model') -> None:
+        '''
+        Save this Evolution Python object/model in a pickle file.
+        Also removes non-essential data to reduce file size.
+        '''
+        # Clearing or shrinking these attributes provides a smaller file size.
+        self.best_creatures[-10:]
+        self.creatures = [self.best_creature]
+        self.all_data = None
+        self.standardized_all_data = None
+        self.standardizer.all_data = None
+        self.standardizer.standardized_data = None
+
+        with open(filename if '.' in filename else filename + '.pkl', 'wb') as f:
+            pickle.dump(self, f)
+
+
+    @classmethod
+    def load(cls, filename):
+        '''
+        Load an Evolution object from a saved, pickle file.
+        '''
+        try:
+            with open(filename, 'rb') as f:
+                return pickle.load(f)
+        except:
+            with open(filename + '.pkl', 'rb') as f:
+                return pickle.load(f)
+
+
     def predict(self, data: Union[Dict[str, float], List[Dict[str, float]], DataFrame], prediction_key: str='', standardized_data: bool=False):
         '''
         Add best_creature predictions to data arg as f'{target}_PREDICTED' new key.
@@ -471,7 +523,7 @@ class Evolution(BaseEvolution):
 
 
 
-def find_best_creature(creatures: list, target_parameter: str, data: list, standardizer=None, progressbar=True) -> tuple:
+def find_best_creature(creatures: list, target_parameter: str, data: list, standardizer=None, progressbar=True, cpu_index=0) -> tuple:
     '''
     Determines best EvogressionCreature and returns misc objects (more than otherwise needed)
     so that multiprocessing (easy_multip) can be used.
@@ -488,7 +540,7 @@ def find_best_creature(creatures: list, target_parameter: str, data: list, stand
     data_length = len(data)
 
     best_creature = None
-    iterable = tqdm.tqdm(creatures) if progressbar else creatures
+    iterable = tqdm.tqdm(creatures, position=cpu_index, desc=f'Process {cpu_index+1}', dynamic_ncols=True, leave=False) if progressbar else creatures
     actual_target_values = [data_point[target_parameter] for data_point in data]  # pull these values once instead of each time in loop comp below
     for creature in iterable:
         if not creature.error_sum:
@@ -502,7 +554,6 @@ def find_best_creature(creatures: list, target_parameter: str, data: list, stand
 
     median_error = sorted(errors)[len(errors) // 2]  # MEDIAN error
     return (best_creature, best_error, median_error, calculated_creatures)
-find_best_creature_multip = easy_multip.decorators.use_multip(find_best_creature)
 
 
 
