@@ -15,7 +15,7 @@ random.seed(10)  # for reproducing the same plot
 class Test3DSurfaceRegression(unittest.TestCase):
 
     def test_best_creature_3d(self):
-        evolution = evogression.Evolution('z', surface_3d_data, num_creatures=3000, num_cycles=10)
+        evolution = evogression.Evolution('z', surface_3d_data, num_creatures=30000, num_cycles=10, num_cpu=3)
         z_test = [evolution.predict(d, 'pred')['pred'] for d in surface_3d_data]
 
         fig = plt.figure()
